#pragma once

#include <memory>
#include <QString>
#include <QList>
#include <qHash>
#include <tuple>
#include <QVariant>
#include "argsparser_global.h"

namespace args_parse {
	/**
	 * .
	 */
	enum class ParamType {
		IndexParam,
		FloatParam,
	};

	/**
	 * @brief в.
	 */
	class IArgvPack {
	public:
		virtual ~IArgvPack() = default;
		/**
		 * .
		 * 
		 * \return 
		 */
		virtual ParamType paramType() const = 0;
		/**
		 * @brief .
		 * 
		 * \return 
		 */
		virtual QString means() const = 0;
		virtual QString placeHolder(bool decorate = true) const = 0;
		virtual QVariant value() const = 0;
		/**
		 * @brief ƥ䳤.
		 * 
		 * \return 
		 */
		virtual int matchLenth() const = 0;
		/**
		 * @brief .
		 * 
		 * \param argv 
		 * \param start ʼ
		 * \return ƥɹ
		 */
		virtual bool parse(const QList<QString> args) = 0;
	};

	class ARGSPARSER_EXPORT __ArgvPackImpls : public IArgvPack {
	private:
		QString means_store;
		QVariant value_store;
		ParamType type_store;

	public:
		__ArgvPackImpls(const QString &means, ParamType t);

		void setValue(const QVariant &v);

		// ͨ IArgvPack ̳
		ParamType paramType() const override;
		QString means() const override;
		QVariant value() const override;
	};

	class ARGSPARSER_EXPORT __FloatArgvImpls : public __ArgvPackImpls {
	private:
		QString key_name;
		bool optional_value;

	public:
			explicit __FloatArgvImpls(const QString& key, const QString& means, bool optional);

			QString bindKey() const;
			virtual bool optional() const;
	};

	/**
	 * @brief key-valueƥģʽ.
	 *  --key value
	 */
	class ARGSPARSER_EXPORT FloatKeyValue : public __FloatArgvImpls {
	public:
		/**
		 * key-valueԪ.
		 * 
		 * \param key 
		 * \param means 
		 * \param optional Ƿѡ
		 */
		explicit FloatKeyValue(const QString& key, const QString &means, bool optional = false);
		virtual ~FloatKeyValue() = default;

		virtual QString placeHolder(bool decorate = true) const override;
		virtual int matchLenth() const override;
		bool parse(const QList<QString> args) override;
	};

	/**
	 * @brief keyƥģʽ.
	 * --key
	 */
	class ARGSPARSER_EXPORT FloatOption : public __FloatArgvImpls {
	public:
		/**
		 * optionԪ.
		 *
		 * \param key 
		 * \param means 
		 * \param optional Ƿѡ
		 */
		explicit FloatOption(const QString& key, const QString& means, bool optional = false);
		virtual ~FloatOption() = default;

		virtual QString placeHolder(bool decorate = true) const override;
		virtual int matchLenth() const override;
		bool parse(const QList<QString> args) override;
	};
	
	/**
	 * .
	 */
	class ARGSPARSER_EXPORT IndexParam : public __ArgvPackImpls {
	private:
		QString _place_holder;

	public:
		explicit IndexParam(const QString & place_v, const QString &means);
		virtual ~IndexParam() = default;

		// ͨ __ArgvPackImpls ̳
		virtual QString placeHolder(bool decorate = true) const override;
		int matchLenth() const override;
		bool parse(const QList<QString> args) override;
	};

	class ARGSPARSER_EXPORT MatchMode {
	private:
		int code_store;
		QString _means_explain;

		QList<std::shared_ptr<IArgvPack>> args_mode;

	public:
		explicit MatchMode(int mode_code, const QString &explain);

		/**
		* @brief ȡģʽ
		* @return ģʽ
		*/
		int modeCode()const;

		/**
		 * ȡַ.
		 * 
		 * \return 
		 */
		QString usageString() const;

		/**
		 * ģʽַ.
		 * 
		 * \return 
		 */
		QString explanString() const;

		/**
		 * ӲԪ.
		 * 
		 * \param unit
		 * \return 
		 */
		MatchMode& operator<<(std::shared_ptr<IArgvPack> unit);

		/**
		* @brief ƥ
		*/
		bool parse(const QList<QString>& args);

		/**
		 * ȡԪ.
		 *
		 * \param key
		 * \return
		 */
		std::shared_ptr<IArgvPack> getUnitViaKey(const QString& key);
		std::shared_ptr<IArgvPack> getUnitViaInitIndex(int idx);
	};

	class ARGSPARSER_EXPORT ArgsParser {
	private:
		QList<std::shared_ptr<MatchMode>> match_modes;

	public:
		QString helperDoc() const;

		ArgsParser& operator<<(std::shared_ptr<MatchMode> mode);

		std::shared_ptr<MatchMode> parse(int argc, char* argv[]);
	};

}